#ifndef DUNE_TECTONIC_GLOBAL_NONLINEARITY_HH
#define DUNE_TECTONIC_GLOBAL_NONLINEARITY_HH

#include <dune/common/fvector.hh>
#include <dune/common/shared_ptr.hh>
#include <dune/istl/bvector.hh>
#include <dune/istl/bcrsmatrix.hh>

#include "nicefunction.hh"
#include "localnonlinearity.hh"

namespace Dune {
template <int dim,
          class VectorTypeTEMPLATE = BlockVector<FieldVector<double, dim>>,
          class MatrixTypeTEMPLATE = BCRSMatrix<FieldMatrix<double, dim, dim>>>
class GlobalNonlinearity {
public:
  typedef VectorTypeTEMPLATE VectorType;
  typedef MatrixTypeTEMPLATE MatrixType;

  /*
    Return a restriction of the outer function to the i'th node.
  */
  virtual shared_ptr<LocalNonlinearity<dim> const> restriction(int i) const = 0;

  void addHessian(VectorType const &v, MatrixType &hessian) const {
    // TODO: is this correct?
    for (size_t i = 0; i < v.size(); ++i) {
      auto res = restriction(i);
      res->addHessian(v[i], hessian[i][i]);
    }
  }

  void directionalDomain(VectorType const &, VectorType const &,
                         Interval<double> &dom) const {
    dom[0] = -std::numeric_limits<double>::max();
    dom[1] = std::numeric_limits<double>::max();
  }

  void directionalSubDiff(VectorType const &u, VectorType const &v,
                          Interval<double> &subdifferential) const {
    subdifferential[0] = subdifferential[1] = 0;
    for (size_t i = 0; i < u.size(); ++i) {
      Interval<double> D;
      auto res = restriction(i);
      // TODO: is the coordinate system the right one here?
      res->directionalSubDiff(u[i], v[i], D);
      subdifferential[0] += D[0];
      subdifferential[1] += D[1];
    }
  }

  void addHessianIndices(Dune::MatrixIndexSet &indices) const {
    for (size_t i = 0; i < indices.rows(); ++i)
      indices.add(i, i);
  }

  void addGradient(VectorType const &v, VectorType &gradient) const {
    for (size_t i = 0; i < v.size(); ++i) {
      auto res = restriction(i);
      res->addGradient(v[i], gradient[i]);
    }
  }

  double regularity(int i, typename VectorType::block_type const &x) const {
    auto res = restriction(i);
    return res->regularity(x);
  }
};
}
#endif
